@@ -5,6 +5,7 @@ import os
 import signal
 import subprocess
 import sys
+import time
 from os.path import join
 from threading import Condition
 from threading import Event as TEvent
@@ -27,17 +28,18 @@ from .plugins import (
 from .plugins.settings import (
     Validator,
     get_default_state,
+    get_settings_hash,
     load_blacklist_yaml,
     load_profile_yaml,
     load_state_yaml,
     merge_settings,
+    parse_defaults,
     save_blacklist_yaml,
     save_profile_yaml,
-    get_settings_hash,
     save_state_yaml,
     validate_config,
 )
-from .utils import expanduser, fix_perms, get_context, switch_priviledge
+from .utils import expanduser, fix_perms, get_context, get_os, switch_priviledge
 
 logger = logging.getLogger(__name__)
 
@@ -46,12 +48,14 @@ CONFIG_DIR = os.environ.get("HHD_CONFIG_DIR", "~/.config/hhd")
 ERROR_DELAY = 5
 INIT_DELAY = 0.4
 POLL_DELAY = 2
+SLEEP_MIN_T = 8
 
 
 class EmitHolder(Emitter):
-    def __init__(self, condition: Condition) -> None:
+    def __init__(self, condition: Condition, ctx) -> None:
         self._events = []
         self._condition = condition
+        super().__init__(ctx=ctx)
 
     def __call__(self, event: Event | Sequence[Event]) -> None:
         with self._condition:
@@ -83,6 +87,14 @@ def notifier(ev: TEvent, cond: Condition):
     return _inner
 
 
+def get_wakeup_count():
+    try:
+        with open("/sys/power/wakeup_count", "r") as f:
+            return int(f.read().strip())
+    except Exception:
+        return -1
+
+
 def print_token(ctx):
     token_fn = expanduser(join(CONFIG_DIR, "token"), ctx)
 
@@ -135,6 +147,8 @@ def main():
     https = None
     prev_http_cfg = None
     updated = False
+    last_event = None
+    info = Config()
 
     # Check we are in a virtual environment
     # TODO: Improve
@@ -171,6 +185,9 @@ def main():
             else:
                 logger.error(f"Command '{args.command[0]}' is unknown. Ignoring...")
 
+        # Get OS Info
+        info["os"] = get_os()
+
         # Use blacklist
         blacklist_fn = join(hhd_dir, "plugins.yml")
         blacklist = load_blacklist_yaml(blacklist_fn)
@@ -217,10 +234,26 @@ def main():
             logger.error(f"No plugins started, exiting...")
             return
 
+        # Load locales
+        locales = []
+        for register in pkg_resources.iter_entry_points("hhd.i18n"):
+            locales.extend(register.resolve()())
+        locales.sort(key=lambda x: x["priority"], reverse=True)
+
+        if locales:
+            lstr = "Loaded the following locales:\n"
+            for locale in locales:
+                lstr += (
+                    f" - {locale['domain']} ({locale['priority']}): {locale['dir']}\n"
+                )
+            logger.info(lstr[:-1])
+        else:
+            logger.info("No locales found.")
+
         # Open plugins
         lock = RLock()
         cond = Condition(lock)
-        emit = EmitHolder(cond)
+        emit = EmitHolder(cond, ctx)
         for p in sorted_plugins:
             set_log_plugin(getattr(p, "log") if hasattr(p, "log") else "ukwn")
             p.open(emit, ctx)
@@ -244,11 +277,15 @@ def main():
         # Monitor config files for changes
         should_initialize = TEvent()
         initial_run = True
+        reset = False
         should_exit = TEvent()
         signal.signal(signal.SIGPOLL, notifier(should_initialize, cond))
         signal.signal(signal.SIGINT, notifier(should_exit, cond))
         signal.signal(signal.SIGTERM, notifier(should_exit, cond))
 
+        # Get wakeup count for sleep detection
+        wakeup_count = get_wakeup_count()
+
         while not should_exit.is_set():
             #
             # Configuration
@@ -275,26 +312,64 @@ def main():
                 settings = merge_settings(
                     [*[p.settings() for p in sorted_plugins], hhd_settings]
                 )
+                # Force general settings to be last
+                if "hhd" in settings:
+                    settings = dict(settings)
+                    tmp = settings.pop("hhd")
+                    settings["hhd"] = tmp
                 shash = get_settings_hash(settings)
 
                 # State
-                new_conf = load_state_yaml(state_fn, settings)
-                if not new_conf:
-                    if conf.conf:
-                        logger.warning(f"Using previous configuration.")
-                    else:
-                        logger.info(f"Using default configuration.")
-                        conf = get_default_state(settings)
+                if reset:
+                    logger.warning(f"Resetting settings.")
+                    conf = get_default_state(settings)
+                    conf.updated = True
+                    reset = False
                 else:
-                    conf = new_conf
+                    new_conf = load_state_yaml(state_fn, settings)
+                    if not new_conf:
+                        if conf.conf:
+                            logger.warning(f"Using previous configuration.")
+                        else:
+                            logger.info(f"Using default configuration.")
+                            conf = get_default_state(settings)
+                    else:
+                        conf = new_conf
+
+                from importlib.metadata import version
 
                 try:
-                    from importlib.metadata import version
-
-                    conf["hhd.settings.version"] = version("hhd")
+                    ver = version("hhd")
+                    conf["hhd.settings.version"] = ver
+                    logger.info(f"Handheld Daemon Version: {ver}")
                 except Exception:
                     pass
 
+                try:
+                    ver = version("adjustor")
+                    conf["hhd.settings.version_adj"] = ver
+                    logger.info(f"Adjustor Version: {ver}")
+                except Exception:
+                    conf["hhd.settings.version_adj"] = "Not Installed"
+                    logger.info(f"Adjustor not installed")
+
+                try:
+                    from hhd.plugins.overlay.overlay import (
+                        find_overlay_exe,
+                        get_overlay_version,
+                    )
+
+                    exe = find_overlay_exe(ctx)
+                    if exe:
+                        ver = get_overlay_version(exe, ctx)
+                        conf["hhd.settings.version_ui"] = ver
+                        logger.info(f"Overlay Version: {ver}")
+                    else:
+                        conf["hhd.settings.version_ui"] = "Not Installed"
+                        logger.info(f"Overlay not installed")
+                except Exception:
+                    logger.info(exe)
+
                 # Profiles
                 profiles = {}
                 templates = {}
@@ -383,8 +458,16 @@ def main():
 
                     set_log_plugin("rest")
                     https = HHDHTTPServer(localhost, port, token)
-                    https.update(settings, conf, profiles, emit)
-                    https.open()
+                    https.update(settings, conf, info, profiles, emit, locales, ctx)
+                    try:
+                        https.open()
+                    except Exception as e:
+                        logger.error(
+                            f"Could not start http API on port {port}.\n"
+                            + "Is another version of Handheld Daemon open?\n"
+                            + "Closing."
+                        )
+                        return
                     update_log_plugins()
                     set_log_plugin("main")
 
@@ -393,8 +476,37 @@ def main():
             #
 
             # Process events
+            set_log_plugin("main")
             settings_changed = False
-            for ev in emit.get_events():
+            events = emit.get_events()
+
+            new_wakeup_count = get_wakeup_count()
+            curr = time.time()
+            # Debounce sleep event to avoid spurious wakeup triggers
+            # This loop will run every 2 seconds, perhaps 4 seconds if there is
+            # a delay. Unless 8 seconds lapse, ignore the event
+            if (
+                new_wakeup_count != wakeup_count
+                and last_event
+                and curr > last_event + SLEEP_MIN_T
+            ):
+                logger.info(
+                    f"System woke up from sleep. Wakeup count: {new_wakeup_count} from {wakeup_count}."
+                )
+                events: Sequence[Event] = [
+                    *events,
+                    {
+                        "type": "special",
+                        "event": "wakeup",
+                        "data": {
+                            "count": new_wakeup_count
+                        },  # FIXME: Count might be removed in the future
+                    },
+                ]
+            wakeup_count = new_wakeup_count
+            last_event = curr
+
+            for ev in events:
                 match ev["type"]:
                     case "settings":
                         settings_changed = True
@@ -418,39 +530,70 @@ def main():
                             conf.update(profiles[ev["name"]].conf)
                     case "state":
                         conf.update(ev["config"].conf)
+                    case "special" | "acpi" | "tdp" | "ppd" | "energy":
+                        pass
                     case other:
                         logger.error(f"Invalid event type submitted: '{other}'")
 
-            # Validate config
-            validate_config(conf, settings, validator)
-
             # If settings changed, the configuration needs to reload
             # but it needs to be saved first
             if settings_changed:
-                should_initialize.set()
+                logger.info(f"Reloading settings.")
 
-            # Plugins are promised that once they emit a
-            # settings change they are not called with the old settings
-            if not settings_changed:
-                #
-                # Plugin event loop
-                #
+                # Settings
+                hhd_settings = {"hhd": load_relative_yaml("settings.yml")}
+                # TODO: Improve check
+                try:
+                    if "venv" not in exe_python:
+                        del hhd_settings["hhd"]["settings"]["children"]["update_stable"]
+                        del hhd_settings["hhd"]["settings"]["children"]["update_beta"]
+                except Exception as e:
+                    logger.warning(f"Could not hide update settings. Error:\n{e}")
+                settings = merge_settings(
+                    [*[p.settings() for p in sorted_plugins], hhd_settings]
+                )
+                # Force general settings to be last
+                if "hhd" in settings:
+                    settings = dict(settings)
+                    tmp = settings.pop("hhd")
+                    settings["hhd"] = tmp
+                shash = get_settings_hash(settings)
 
-                for p in reversed(sorted_plugins):
-                    set_log_plugin(getattr(p, "log") if hasattr(p, "log") else "ukwn")
-                    p.prepare(conf)
-                    update_log_plugins()
+                # Add new defaults
+                conf = Config([parse_defaults(settings), conf.conf])
+                conf.updated = True
 
+            # Validate config
+            validate_config(conf, settings, validator)
+
+            #
+            # Plugin event loop
+            #
+
+            # Allow plugins to process events
+            if events:
                 for p in sorted_plugins:
                     set_log_plugin(getattr(p, "log") if hasattr(p, "log") else "ukwn")
-                    p.update(conf)
+                    p.notify(events)
                     update_log_plugins()
-                set_log_plugin("ukwn")
+
+            # Run prepare loop
+            for p in reversed(sorted_plugins):
+                set_log_plugin(getattr(p, "log") if hasattr(p, "log") else "ukwn")
+                p.prepare(conf)
+                update_log_plugins()
+
+            # Run update loop
+            for p in sorted_plugins:
+                set_log_plugin(getattr(p, "log") if hasattr(p, "log") else "ukwn")
+                p.update(conf)
+                update_log_plugins()
+            set_log_plugin("ukwn")
 
             # Notify that events were applied
             # Before saving to reduce delay (yaml files take 100ms :( )
             if https:
-                https.update(settings, conf, profiles, emit)
+                https.update(settings, conf, info, profiles, emit, locales, ctx)
 
             #
             # Save loop
@@ -484,15 +627,16 @@ def main():
                             f"Failed removing profile {name} at:\n{fn}\nWith error:\n{e}"
                         )
 
-            # Add template config
-            if save_profile_yaml(
-                join(profile_dir, "_template.yml"),
-                settings,
-                templates.get("_template", None),
-                shash,
-            ):
-                fix_perms(join(profile_dir, "_template.yml"), ctx)
-                saved = True
+            # Causes unnecessary writes, is not used anyway.
+            # # Add template config
+            # if save_profile_yaml(
+            #     join(profile_dir, "_template.yml"),
+            #     settings,
+            #     templates.get("_template", None),
+            #     shash,
+            # ):
+            #     fix_perms(join(profile_dir, "_template.yml"), ctx)
+            #     saved = True
 
             if not has_new and saved:
                 # We triggered the interrupt, clear
@@ -518,6 +662,7 @@ def main():
                                 "uninstall",
                                 "-y",
                                 "hhd",
+                                "adjustor",
                             ]
                         )
                         subprocess.check_call(
@@ -534,15 +679,62 @@ def main():
                                     if upd_beta
                                     else "hhd"
                                 ),
+                                (
+                                    "git+https://github.com/hhd-dev/adjustor"
+                                    if upd_beta
+                                    else "adjustor"
+                                ),
                             ]
                         )
+
+                        if not upd_beta:
+                            # No beta version for the UI yet, skip updating it
+                            import json
+                            import stat
+                            import urllib.request
+
+                            with urllib.request.urlopen(
+                                "https://api.github.com/repos/hhd-dev/hhd-ui/releases/latest"
+                            ) as f:
+                                release_data = json.load(f)
+
+                            for asset in release_data["assets"]:
+                                os.makedirs(
+                                    expanduser("~/.local/bin", ctx), exist_ok=True
+                                )
+                                if "hhd-ui.AppImage" == asset["name"]:
+                                    REPORT_POINTS = 8
+
+                                    def progress(idx, blockSize, total):
+                                        interval = int(
+                                            total / blockSize / REPORT_POINTS
+                                        )
+                                        if idx % interval == 0:
+                                            logger.info(
+                                                f"Downloading overlay: {100*idx*blockSize / total:.1f}%"
+                                            )
+
+                                    out_fn = expanduser("~/.local/bin/hhd-ui", ctx)
+                                    urllib.request.urlretrieve(
+                                        asset["browser_download_url"],
+                                        out_fn,
+                                        reporthook=progress,
+                                    )
+
+                                    st = os.stat(out_fn)
+                                    os.chmod(out_fn, st.st_mode | stat.S_IEXEC)
+                                    break
+
+                        # Set updated
                         updated = True
                     else:
                         logger.error(
                             f"Could not update, python executable is not within a venv (checked for 'venv' in path name):\n{exe_python}"
                         )
                 except Exception as e:
-                    logger.error(f"Error while updating:\n{e}")
+                    err = f"Error while updating:\n{e}"
+                    conf["hhd.settings.update_error"] = err
+                    logger.error(err)
                 switch_priviledge(ctx, True)
 
                 if updated:
@@ -558,6 +750,12 @@ def main():
                 ):
                     cond.wait(timeout=POLL_DELAY)
 
+            # Check reset
+            if conf["hhd.settings.reset"].to(bool):
+                conf["hhd.settings.reset"] = False
+                should_initialize.set()
+                reset = True
+
         set_log_plugin("main")
         logger.info(f"Received interrupt or updated. Stopping plugins and exiting.")
     finally:
@@ -577,6 +775,17 @@ def main():
                 set_log_plugin(getattr(p, "log") if hasattr(p, "log") else "ukwn")
                 p.close()
 
+        set_log_plugin("main")
+        try:
+            logger.info("Closing cached controllers.")
+            from hhd.controller.virtual.dualsense import Dualsense
+            from hhd.controller.virtual.uinput import UInputDevice
+
+            UInputDevice.close_cached()
+            Dualsense.close_cached()
+        except Exception as e:
+            logger.error("Could not close cached controllers with error:\n{e}")
+
     if updated:
         # Use error code to restart service
         sys.exit(-1)
