@@ -2,6 +2,7 @@ import glob
 import logging
 import os
 import re
+import stat
 import subprocess
 import time
 from typing import Collection, Mapping, Sequence, TypeVar, cast
@@ -13,7 +14,7 @@ from hhd.controller import Axis, Button, Consumer, Event, Producer, can_read
 from hhd.controller.base import Event
 from hhd.controller.const import AbsAxis, GamepadButton, KeyboardButton
 from hhd.controller.lib.common import hexify, matches_patterns
-from hhd.controller.lib.hide import hide_gamepad, unhide_all, unhide_gamepad
+from hhd.controller.lib.hide import hide_gamepad, unhide_gamepad
 
 logger = logging.getLogger(__name__)
 
@@ -98,6 +99,34 @@ DINPUT_AXIS_POSTPROCESS = {
     "rs_y": {"zero_is_middle": True},
 }
 
+if calib := os.environ.get("HHD_CALIB"):
+    # TODO: Move this into the gui
+    # Accepts the following format: <axis>: <min> <max> <deadzone>
+    # Everything below deadzone gets set to 0.
+    # If axis is positive, it gets scaled so that it is 1 when max is hit
+    # If negative, it is scaled to be -1 when it reaches the min value
+    # HHD_CALIB=$(cat <<-END
+    # {
+    #     "ls_x": {"min": -1, "max": 1, "deadzone": 0.05},
+    #     "ls_y": {"min": -1, "max": 1, "deadzone": 0.05},
+    #     "rs_x": {"min": -1, "max": 1, "deadzone": 0.05},
+    #     "rs_y": {"min": -1, "max": 1, "deadzone": 0.05},
+    #     "lt": {"min": 0, "max": 1, "deadzone": 0.05},
+    #     "rt": {"min": 0, "max": 1, "deadzone": 0.05}
+    # }
+    # END
+    # )
+    import json
+
+    try:
+        AXIS_CALIBRATION = json.loads(calib)
+        logger.info(f"Loaded calibration:\n{calib}")
+    except Exception as e:
+        logger.info(f"Could not load Axis Calibration:\n{calib}\nError:{e}")
+        AXIS_CALIBRATION = {}
+else:
+    AXIS_CALIBRATION = {}
+
 
 def list_joysticks(input_device_dir="/dev/input"):
     return glob.glob(f"{input_device_dir}/js*")
@@ -121,7 +150,83 @@ def find_joystick(ev: str):
             return other
 
 
+def is_device(fn):
+    """Check if ``fn`` is a readable and writable character device."""
+
+    if not os.path.exists(fn):
+        return False
+
+    m = os.stat(fn)[stat.ST_MODE]
+    if not stat.S_ISCHR(m):
+        return False
+
+    if not os.access(fn, os.R_OK | os.W_OK):
+        return False
+
+    return True
+
+
+def list_evs(filter_valid: bool = False, fn: str = "/proc/bus/input/devices"):
+    with open(fn, "r") as f:
+        data = f.read()
+
+    devs = {}
+    for d in data.split("\n\n"):
+        out = {}
+        out["hash"] = hash(d)
+        for line in d.split("\n"):
+            if not line:
+                continue
+            match line[0]:
+                case "I":
+                    for attr in line[3:-1].split(" "):
+                        name, val = attr.split("=")
+                        out[name.lower()] = int(val, 16)
+                case "N":
+                    out["name"] = line[len('N: Name="') : -1]
+                case "B":
+                    if "byte" not in out:
+                        out["byte"] = {}
+                    head, raw = line[3:].split("=")
+                    arr = bytearray()
+                    for x in raw.split(" "):
+                        if not x:
+                            continue
+                        arr.extend(int(x, 16).to_bytes(8, "big"))
+                    # Array is stacked using big endianness, so
+                    # we reverse it to little endian
+                    out["byte"][head.lower()] = bytes(reversed(arr))
+                case "P":
+                    out["phys"] = line[len('P: Phys="') : -1]
+                case "S":
+                    if "Sysfs" in line:
+                        out["sysfs"] = line[len('S: Sysfs="') : -1]
+                case "H":
+                    if len(line) < len("H: Handlers=") + 1:
+                        continue
+                    for handler in line[len("H: Handlers=") : -1].split(" "):
+                        if "event" in handler:
+                            pth = "/dev/input/" + handler
+                            if not filter_valid or is_device(pth):
+                                devs[pth] = out
+
+    return devs
+
+
+def enumerate_evs(
+    vid: int | None = None, pid: int | None = None, filter_valid: bool = False
+):
+    evs = list_evs(filter_valid)
+    return {
+        k: v
+        for k, v in evs.items()
+        if (vid is None or vid == v.get("vendor", None))
+        and (pid is None or pid == v.get("product", None))
+    }
+
+
 class GenericGamepadEvdev(Producer, Consumer):
+
     def __init__(
         self,
         vid: Sequence[int],
@@ -136,7 +241,7 @@ class GenericGamepadEvdev(Producer, Consumer):
         grab: bool = True,
         msc_map: Mapping[int, Button] = {},
         msc_delay: float = 0.1,
-        postprocess: dict[str, dict] = {},
+        postprocess: dict[str, dict] = AXIS_CALIBRATION,
     ) -> None:
         self.vid = vid
         self.pid = pid
@@ -154,19 +259,20 @@ class GenericGamepadEvdev(Producer, Consumer):
         self.required = required
         self.hide = hide
         self.grab = grab
-        self.hidden = False
+        self.hidden = None
         self.queue = []
         self.postprocess = postprocess
+        self.start_pressed = None
 
     def open(self) -> Sequence[int]:
-        for d in evdev.list_devices():
+        for d, info in list_evs(filter_valid=True).items():
+            if not matches_patterns(info.get("vendor", ""), self.vid):
+                continue
+            if not matches_patterns(info.get("product", ""), self.pid):
+                continue
+            if not matches_patterns(info.get("name", ""), self.name):
+                continue
             dev = evdev.InputDevice(d)
-            if not matches_patterns(dev.info.vendor, self.vid):
-                continue
-            if not matches_patterns(dev.info.product, self.pid):
-                continue
-            if not matches_patterns(dev.name, self.name):
-                continue
             if self.capabilities:
                 matches = True
                 dev_cap = cast(dict[int, Sequence[int]], dev.capabilities())
@@ -181,18 +287,8 @@ class GenericGamepadEvdev(Producer, Consumer):
                 if not matches:
                     continue
 
-            self.dev = dev
-            if self.grab:
-                self.dev.grab()
-            self.ranges = {
-                a: (i.min, i.max) for a, i in self.dev.capabilities().get(B("EV_ABS"), [])  # type: ignore
-            }
-            self.fd = dev.fd
-            self.started = True
-            self.effect_id = -1
-            self.queue = []
-
-            # Run after init to avoid having leftover rules
+            # hide_gamepad will destroy the current fds, so run it before
+            # creating the final device
             if self.hide:
                 # Check we are root
                 if not os.getuid():
@@ -206,6 +302,27 @@ class GenericGamepadEvdev(Producer, Consumer):
                         f"Not running as root, device '{dev.name}' could not be hid."
                     )
 
+            try:
+                # Close the previous device
+                # Will have been destroyed by hiding
+                dev.close()
+                self.dev = evdev.InputDevice(d)
+                if self.grab:
+                    self.dev.grab()
+                self.ranges = {
+                    a: (i.min, i.max) for a, i in self.dev.capabilities().get(B("EV_ABS"), [])  # type: ignore
+                }
+                self.supports_vibration = B("EV_FF") in dev.capabilities()
+                self.fd = self.dev.fd
+                self.started = True
+                self.effect_id = -1
+                self.queue = []
+            except Exception as e:
+                # Prevent leftover rules in case of error
+                if self.hidden:
+                    unhide_gamepad(d, self.hidden)
+                raise e
+
             return [self.fd]
 
         err = f"Device with the following not found:\n"
@@ -224,8 +341,8 @@ class GenericGamepadEvdev(Producer, Consumer):
 
     def close(self, exit: bool) -> bool:
         if self.dev:
-            if self.hidden:
-                unhide_gamepad(self.dev.path)
+            if self.hidden and exit:
+                unhide_gamepad(self.dev.path, self.hidden)
             self.dev.close()
             self.dev = None
             self.fd = 0
@@ -238,6 +355,9 @@ class GenericGamepadEvdev(Producer, Consumer):
         for ev in events:
             match ev["type"]:
                 case "rumble":
+                    if not self.supports_vibration:
+                        continue
+
                     # Erase old effect
                     if self.effect_id != -1:
                         self.dev.erase_effect(self.effect_id)
@@ -274,6 +394,15 @@ class GenericGamepadEvdev(Producer, Consumer):
             if curr >= t:
                 out.append(ev)
                 self.queue.pop(0)
+        if self.start_pressed and curr - self.start_pressed > 0.07:
+            self.start_pressed = None
+            out.append(
+                {
+                    "type": "button",
+                    "code": self.btn_map[B("KEY_LEFTMETA")],
+                    "value": True,
+                }
+            )
 
         if not self.dev or not self.fd in fds:
             return out
@@ -293,7 +422,14 @@ class GenericGamepadEvdev(Producer, Consumer):
                 if e.type == B("EV_KEY"):
                     if e.code in self.btn_map:
                         # Only 1 is valid for press (look at sysrq)
-                        if e.value == 0 or e.value == 1:
+                        if e.code == B("KEY_LEFTMETA") and e.value:
+                            # start requires special handling
+                            # If it exists on the button map, it may
+                            # also be used for other shortcuts.
+                            # So we have to wait a bit to see if it is
+                            # a standalone press
+                            self.start_pressed = curr
+                        elif e.value == 0 or e.value == 1:
                             out.append(
                                 {
                                     "type": "button",
@@ -301,6 +437,7 @@ class GenericGamepadEvdev(Producer, Consumer):
                                     "value": bool(e.value),
                                 }
                             )
+                            self.start_pressed = None
                 elif e.type == B("EV_ABS"):
                     if e.code in self.axis_map:
                         ax = self.axis_map[e.code]
@@ -315,6 +452,23 @@ class GenericGamepadEvdev(Producer, Consumer):
                                 self.ranges[e.code][1 if e.value >= 0 else 0]
                             )
 
+                        # Calibrate
+                        if ax in self.postprocess:
+                            calib = self.postprocess[ax]
+                            if val < 0 and "min" in calib:
+                                m = calib["min"]
+                                if m:
+                                    # avoid division by 0
+                                    val = -max(m, val) / m
+                            elif val > 0 and "max" in calib:
+                                m = calib["max"]
+                                val = min(m, val) / m
+
+                            if "deadzone" in calib:
+                                d = calib["deadzone"]
+                                if abs(val) < d:
+                                    val = 0
+
                         out.append(
                             {
                                 "type": "axis",
