@@ -32,30 +32,31 @@ class DeviceInfo(NamedTuple):
 
 ACCEL_NAMES = ["accel_3d"]
 GYRO_NAMES = ["gyro_3d"]
-IMU_NAMES = ["bmi323-imu", "BMI0160", "BMI0260"]
+# FIXME: this :00 needs to be cleaned up
+IMU_NAMES = ["bmi323-imu", "BMI0160", "BMI0260", "i2c-10EC5280:00", "i2c-BOSC0260:00"]
 SYSFS_TRIG_CONFIG_DIR = os.environ.get("HHD_MOUNT_TRIG_SYSFS", "/var/trig_sysfs_config")
 
 ACCEL_MAPPINGS: dict[str, tuple[Axis, str | None, float, float | None]] = {
-    "accel_x": ("accel_z", "accel", 1, 3),
-    "accel_y": ("accel_x", "accel", 1, 3),
-    "accel_z": ("accel_y", "accel", 1, 3),
+    "accel_x": ("accel_z", "accel", 1, None),
+    "accel_y": ("accel_x", "accel", 1, None),
+    "accel_z": ("accel_y", "accel", 1, None),
     "timestamp": ("accel_ts", None, 1, None),
 }
 GYRO_MAPPINGS: dict[str, tuple[Axis, str | None, float, float | None]] = {
     "anglvel_x": ("gyro_z", "anglvel", 1, None),
     "anglvel_y": ("gyro_x", "anglvel", 1, None),
     "anglvel_z": ("gyro_y", "anglvel", 1, None),
-    "timestamp": ("gyro_ts", None, 1, None),
+    "timestamp": ("imu_ts", None, 1, None),
 }
 
 BMI_MAPPINGS: dict[str, tuple[Axis, str | None, float, float | None]] = {
-    "accel_x": ("accel_z", "accel", 1, 3),
-    "accel_y": ("accel_x", "accel", 1, 3),
-    "accel_z": ("accel_y", "accel", 1, 3),
-    "anglvel_x": ("gyro_z", "anglvel", 1, None),
+    "accel_x": ("accel_z", "accel", -1, None),
+    "accel_y": ("accel_x", "accel", 1, None),
+    "accel_z": ("accel_y", "accel", -1, None),
+    "anglvel_x": ("gyro_z", "anglvel", -1, None),
     "anglvel_y": ("gyro_x", "anglvel", 1, None),
     "anglvel_z": ("gyro_y", "anglvel", -1, None),
-    "timestamp": ("gyro_ts", None, 1, None),
+    "timestamp": ("imu_ts", None, 1, None),
 }
 
 
@@ -252,6 +253,10 @@ class IioReader(Producer):
         )
 
         if not dev:
+            logger.error(
+                "IMU not found for this device, gyro will not work.\n"
+                + "You need to install the IMU driver for your device, see the readme."
+            )
             return []
 
         self.buf = None
@@ -295,14 +300,10 @@ class IioReader(Producer):
             if se.axis:
                 # TODO: Implement parsing iio fully, by adding shifting and cutoff
                 d = data[ofs >> 3 : (ofs >> 3) + (se.storage_bits >> 3)]
-                d = int.from_bytes(d, byteorder=se.endianness, signed=se.signed)
-                if self.legion_fix and d == -124:
-                    # Legion go likes to overflow to -124 in both directions
-                    # skip this number to avoid jitters
-                    continue
+                d_raw = int.from_bytes(d, byteorder=se.endianness, signed=se.signed)
                 # d = d >> se.shift
                 # d &= (1 << se.bits) - 1
-                d = d * se.scale + se.offset
+                d = d_raw * se.scale + se.offset
 
                 if se.max_val is not None:
                     if d > 0:
@@ -311,13 +312,27 @@ class IioReader(Producer):
                         d = max(d, -se.max_val)
 
                 if se.axis not in self.prev or self.prev[se.axis] != d:
-                    out.append(
-                        {
-                            "type": "axis",
-                            "code": se.axis,
-                            "value": d,
-                        }
-                    )
+                    if not (
+                        self.legion_fix and (d_raw == -124 or d_raw // 1000 == -125)
+                    ):
+                        # Legion go likes to overflow to -124 in both directions
+                        # skip this number to avoid jitters
+                        # With a kernel patch to allow higher resolution, this happens
+                        # with the following numbers
+                        # 4d 95 f3 c7: -124715
+                        # 33 97 f3 c7: -124718
+                        # Reported by hhd: -124422, -124419
+                        # Legion go axis tester
+                        # import time
+                        # if se.axis == "gyro_x":
+                        #     print(f"{time.time() % 1:.3f} {d_raw}")
+                        out.append(
+                            {
+                                "type": "axis",
+                                "code": se.axis,
+                                "value": d,
+                            }
+                        )
                     self.prev[se.axis] = d
             ofs += se.storage_bits
 
@@ -475,6 +490,7 @@ class HrtimerTrigger(IioReader):
             return False
 
         self.old_triggers = {}
+        found = False
         for d in self.devices:
             s, _ = find_sensor(d)
             if not s:
@@ -488,6 +504,15 @@ class HrtimerTrigger(IioReader):
                 self.old_triggers[trig_fn] = (f.read(), buff_fn)
             with open(trig_fn, "w") as f:
                 f.write(f"hhd")
+            found = True
+
+        if not found:
+            self.close()
+            logger.error(
+                "IMU not found for this device, gyro will not work.\n"
+                + "You need to install the IMU driver for your device, see the readme."
+            )
+            return False
 
         return True
 
