@@ -1,7 +1,7 @@
 import logging
 import time
 from collections import defaultdict
-from typing import Literal, NamedTuple, Sequence, cast
+from typing import Sequence, cast, Literal
 
 from hhd.controller import (
     Consumer,
@@ -12,6 +12,7 @@ from hhd.controller import (
 )
 from hhd.controller.lib.common import encode_axis, set_button
 from hhd.controller.lib.uhid import BUS_BLUETOOTH, BUS_USB, UhidDevice
+from hhd.controller.lib.ccache import ControllerCache
 
 from .const import (
     DS5_BT_AXIS_MAP,
@@ -23,8 +24,9 @@ from .const import (
     DS5_EDGE_MAX_REPORT_FREQ,
     DS5_EDGE_MIN_REPORT_FREQ,
     DS5_EDGE_NAME,
-    DS5_NAME,
     DS5_EDGE_PRODUCT,
+    DS5_EDGE_REPORT_PAIRING,
+    DS5_EDGE_REPORT_PAIRING_ID,
     DS5_EDGE_STOCK_REPORTS,
     DS5_EDGE_TOUCH_HEIGHT,
     DS5_EDGE_TOUCH_WIDTH,
@@ -33,9 +35,11 @@ from .const import (
     DS5_INPUT_CRC32_SEED,
     DS5_INPUT_REPORT_BT_OFS,
     DS5_INPUT_REPORT_USB_OFS,
+    DS5_NAME,
     DS5_PRODUCT,
     DS5_USB_AXIS_MAP,
     DS5_USB_BTN_MAP,
+    DS5_NAME_LEFT,
     DS5_VENDOR,
     patch_dpad_val,
     prefill_ds5_report,
@@ -46,11 +50,20 @@ from .const import (
 REPORT_MAX_DELAY = 1 / DS5_EDGE_MIN_REPORT_FREQ
 REPORT_MIN_DELAY = 1 / DS5_EDGE_MAX_REPORT_FREQ
 DS5_EDGE_MIN_TIMESTAMP_INTERVAL = 1500
+MAX_IMU_SYNC_DELAY = 2
 
 logger = logging.getLogger(__name__)
 
+_cache = ControllerCache()
+_cache_left = ControllerCache()
+
 
 class Dualsense(Producer, Consumer):
+    @staticmethod
+    def close_cached():
+        _cache.close()
+        _cache_left.close()
+
     def __init__(
         self,
         touchpad_method: TouchpadCorrectionType = "crop_end",
@@ -60,7 +73,11 @@ class Dualsense(Producer, Consumer):
         enable_touchpad: bool = True,
         enable_rgb: bool = True,
         sync_gyro: bool = False,
-        paddles_to_clicks: bool = False,
+        flip_z: bool = True,
+        paddles_to_clicks: Literal["disabled", "top", "bottom"] = "disabled",
+        controller_id: int = 0,
+        left_motion: bool = False,
+        cache: bool = False,
     ) -> None:
         self.available = False
         self.report = None
@@ -73,7 +90,12 @@ class Dualsense(Producer, Consumer):
         self.enable_touchpad = enable_touchpad
         self.enable_rgb = enable_rgb
         self.sync_gyro = sync_gyro
+        self.flip_z = flip_z
         self.paddles_to_clicks = paddles_to_clicks
+        self.controller_id = controller_id
+        self.left_motion = left_motion
+        self.cache = cache
+        self.last_imu_ts = 0
 
         self.ofs = (
             DS5_INPUT_REPORT_BT_OFS if use_bluetooth else DS5_INPUT_REPORT_USB_OFS
@@ -84,20 +106,48 @@ class Dualsense(Producer, Consumer):
     def open(self) -> Sequence[int]:
         self.available = False
         self.report = bytearray(prefill_ds5_report(self.use_bluetooth))
-        self.dev = UhidDevice(
-            vid=DS5_VENDOR,
-            pid=DS5_EDGE_PRODUCT if self.edge_mode else DS5_PRODUCT,
-            bus=BUS_BLUETOOTH if self.use_bluetooth else BUS_USB,
-            version=DS5_EDGE_VERSION,
-            country=DS5_EDGE_COUNTRY,
-            name=DS5_EDGE_NAME if self.edge_mode else DS5_NAME,
-            report_descriptor=(
-                DS5_EDGE_DESCRIPTOR_BT
-                if self.use_bluetooth
-                else DS5_EDGE_DESCRIPTOR_USB
-            ),
+
+        cached = cast(
+            Dualsense | None, _cache_left.get() if self.left_motion else _cache.get()
         )
 
+        # Use cached controller to avoid disconnects
+        self.dev = None
+        if cached:
+            if (
+                self.edge_mode == cached.edge_mode
+                and self.use_bluetooth == cached.use_bluetooth
+                and self.controller_id == cached.controller_id
+            ):
+                logger.warning(
+                    f"Using cached controller node for Dualsense {'left motions device' if self.left_motion else 'controller'}."
+                )
+                self.dev = cached.dev
+                if self.dev and self.dev.fd:
+                    self.fd = self.dev.fd
+            else:
+                cached.close(True)
+        name = (
+            (DS5_EDGE_NAME if self.edge_mode else DS5_NAME)
+            if not self.left_motion
+            else DS5_NAME_LEFT
+        )
+        if not self.dev:
+            self.dev = UhidDevice(
+                vid=DS5_VENDOR,
+                pid=DS5_EDGE_PRODUCT if self.edge_mode else DS5_PRODUCT,
+                bus=BUS_BLUETOOTH if self.use_bluetooth else BUS_USB,
+                version=DS5_EDGE_VERSION,
+                country=DS5_EDGE_COUNTRY,
+                name=name,
+                report_descriptor=(
+                    DS5_EDGE_DESCRIPTOR_BT
+                    if self.use_bluetooth
+                    else DS5_EDGE_DESCRIPTOR_USB
+                ),
+            )
+            self.fd = self.dev.open()
+
         self.touch_correction = correct_touchpad(
             DS5_EDGE_TOUCH_WIDTH, DS5_EDGE_TOUCH_HEIGHT, 1, self.touchpad_method
         )
@@ -105,21 +155,26 @@ class Dualsense(Producer, Consumer):
         self.state: dict = defaultdict(lambda: 0)
         self.rumble = False
         self.touchpad_touch = False
-        self.touchpad_down = time.perf_counter()
-        self.start = time.perf_counter_ns()
-        self.fd = self.dev.open()
+        curr = time.perf_counter()
+        self.touchpad_down = curr
+        self.last_imu = curr
+        self.imu_failed = False
+        self.start = time.perf_counter()
 
-        logger.info(
-            f"Starting '{(DS5_EDGE_NAME if self.edge_mode else DS5_NAME).decode()}'."
-        )
+        logger.info(f"Starting '{name.decode()}'.")
+        assert self.fd
         return [self.fd]
 
-    def close(self, exit: bool) -> bool:
-        if not exit:
-            """This is a consumer, so we would deadlock if it was disabled."""
-            return False
-
-        if self.dev:
+    def close(self, exit: bool, in_cache: bool = False) -> bool:
+        if not in_cache and self.cache and time.perf_counter() - self.start:
+            logger.warning(
+                f"Caching Dualsense {'left motions device' if self.left_motion else 'controller'} to avoid reconnection."
+            )
+            if self.left_motion:
+                _cache_left.add(self)
+            else:
+                _cache.add(self)
+        elif self.dev:
             self.dev.send_destroy()
             self.dev.close()
             self.dev = None
@@ -142,7 +197,14 @@ class Dualsense(Producer, Consumer):
                     self.available = False
                 case "get_report":
                     if ev["rnum"] in DS5_EDGE_STOCK_REPORTS:
-                        rep = DS5_EDGE_STOCK_REPORTS[ev["rnum"]]
+                        num = ev["rnum"]
+                        if num == DS5_EDGE_REPORT_PAIRING_ID:
+                            # Customize pairing report to have per-controller
+                            # calibration
+                            rep = DS5_EDGE_REPORT_PAIRING(self.controller_id)
+                        else:
+                            rep = DS5_EDGE_STOCK_REPORTS[num]
+
                         if self.use_bluetooth:
                             rep = sign_crc32_append(rep, DS5_FEATURE_CRC32_SEED)
                         self.dev.send_get_report_reply(ev["id"], 0, rep)
@@ -223,11 +285,19 @@ class Dualsense(Producer, Consumer):
                                 # "brightness": led_brightness / 63
                                 # if led_brightness
                                 # else 1,
-                                "brightness": 1,
+                                "initialize": False,
+                                "direction": "left",
                                 "speed": 0,
+                                "brightness": 1,
+                                "speedd": "high",
+                                "brightnessd": "high",
                                 "red": red,
                                 "blue": blue,
                                 "green": green,
+                                "red2": 0, # disable for OXP
+                                "blue2": 0,
+                                "green2": 0,
+                                "oxp": None,
                             }
                         )
                     # elif (rep[39] & 2) and (rep[42] & 2):
@@ -292,17 +362,36 @@ class Dualsense(Producer, Consumer):
         # To fix gyro to mouse in latest steam
         # only send updates when gyro sends a timestamp
         send = not self.sync_gyro
+        curr = time.perf_counter()
 
         new_rep = bytearray(self.report)
         for ev in events:
+            code = ev["code"]
             match ev["type"]:
                 case "axis":
-                    if not self.enable_touchpad and ev["code"].startswith("touchpad"):
+                    if not self.enable_touchpad and code.startswith("touchpad"):
                         continue
-                    if ev["code"] in self.axis_map:
-                        encode_axis(new_rep, self.axis_map[ev["code"]], ev["value"])
+                    if self.left_motion:
+                        # Only left keep imu events for left motion
+                        if (
+                            "left_gyro_" in code
+                            or "left_accel_" in code
+                            or "left_imu_" in code
+                        ):
+                            code = code.replace("left_", "")
+                        else:
+                            continue
+                    if code in self.axis_map:
+                        if self.flip_z and code == "gyro_z":
+                            ev["value"] = -ev["value"]
+                        try:
+                            encode_axis(new_rep, self.axis_map[code], ev["value"])
+                        except Exception:
+                            logger.warning(
+                                f"Encoding '{ev['code']}' with {ev['value']} overflowed."
+                            )
                     # DPAD is weird
-                    match ev["code"]:
+                    match code:
                         case "hat_x":
                             self.state["hat_x"] = ev["value"]
                             patch_dpad_val(
@@ -341,43 +430,54 @@ class Dualsense(Producer, Consumer):
                                 (y & 0x0F) << 4
                             )
                             new_rep[self.ofs + 35] = y >> 4
-                        case "gyro_ts":
+                        case "gyro_ts" | "accel_ts" | "imu_ts":
                             send = True
+                            self.last_imu = time.perf_counter()
+                            self.last_imu_ts = ev["value"]
                             new_rep[self.ofs + 27 : self.ofs + 31] = int(
                                 ev["value"] / DS5_EDGE_DELTA_TIME_NS
                             ).to_bytes(8, byteorder="little", signed=False)[:4]
                 case "button":
-                    if not self.enable_touchpad and ev["code"].startswith("touchpad"):
+                    if self.left_motion:
+                        # skip buttons for left motion
                         continue
-                    if self.paddles_to_clicks and (ev["code"] == "extra_l1"):
+                    if not self.enable_touchpad and code.startswith("touchpad"):
+                        continue
+                    if (self.paddles_to_clicks == "top" and code == "extra_l1") or (
+                        self.paddles_to_clicks == "bottom" and code == "extra_l2"
+                    ):
                         # Place finger on correct place and click
                         new_rep[self.ofs + 33] = 0x80
                         new_rep[self.ofs + 34] = 0x01
                         new_rep[self.ofs + 35] = 0x20
                         # Replace code with click
                         ev = {**ev, "code": "touchpad_left"}
-                    if self.paddles_to_clicks and (ev["code"] == "extra_r1"):
+                        code = "touchpad_left"
+                    if (self.paddles_to_clicks == "top" and code == "extra_r1") or (
+                        self.paddles_to_clicks == "bottom" and code == "extra_r2"
+                    ):
                         # Place finger on correct place and click
                         new_rep[self.ofs + 33] = 0x00
                         new_rep[self.ofs + 34] = 0x06
                         new_rep[self.ofs + 35] = 0x20
                         # Replace code with click
                         ev = {**ev, "code": "touchpad_left"}
+                        code = "touchpad_left"
 
-                    if ev["code"] in self.btn_map:
-                        set_button(new_rep, self.btn_map[ev["code"]], ev["value"])
+                    if code in self.btn_map:
+                        set_button(new_rep, self.btn_map[code], ev["value"])
 
                     # Fix touchpad click requiring touch
-                    if ev["code"] == "touchpad_touch":
+                    if code == "touchpad_touch":
                         self.touchpad_touch = ev["value"]
-                    if ev["code"] == "touchpad_left":
+                    if code == "touchpad_left":
                         set_button(
                             new_rep,
                             self.btn_map["touchpad_touch"],
                             ev["value"] or self.touchpad_touch,
                         )
                     # Also add right click
-                    if ev["code"] == "touchpad_right":
+                    if code == "touchpad_right":
                         set_button(
                             new_rep,
                             self.btn_map["touchpad_touch"],
@@ -390,7 +490,9 @@ class Dualsense(Producer, Consumer):
                         )
 
                 case "configuration":
-                    match ev["code"]:
+                    if self.left_motion:
+                        continue
+                    match code:
                         case "touchpad_aspect_ratio":
                             self.aspect_ratio = cast(float, ev["value"])
                             self.touch_correction = correct_touchpad(
@@ -415,7 +517,15 @@ class Dualsense(Producer, Consumer):
         #     return
         self.report = new_rep
 
-        if self.fake_timestamps:
+        # If the IMU breaks, smoothly re-enable the controller
+        failover = self.last_imu + MAX_IMU_SYNC_DELAY < curr
+        if self.sync_gyro and failover and not self.imu_failed:
+            self.imu_failed = True
+            logger.error(
+                f"IMU Did not send information for {MAX_IMU_SYNC_DELAY}s. Disabling Gyro Sync."
+            )
+
+        if self.fake_timestamps or failover:
             new_rep[self.ofs + 27 : self.ofs + 31] = int(
                 time.perf_counter_ns() / DS5_EDGE_DELTA_TIME_NS
             ).to_bytes(8, byteorder="little", signed=False)[:4]
@@ -431,5 +541,5 @@ class Dualsense(Producer, Consumer):
 
         if self.use_bluetooth:
             sign_crc32_inplace(self.report, DS5_INPUT_CRC32_SEED)
-        if send:
+        if send or failover:
             self.dev.send_input_report(self.report)
