@@ -3,52 +3,53 @@ from hhd.controller.physical.evdev import B, to_map
 from hhd.plugins import gen_gyro_state
 
 DEFAULT_MAPPINGS: dict[str, tuple[Axis, str | None, float, float | None]] = {
-    "accel_x": ("accel_z", "accel", 1, 3),
-    "accel_y": ("accel_x", "accel", 1, 3),
-    "accel_z": ("accel_y", "accel", 1, 3),
-    "anglvel_x": ("gyro_z", "anglvel", -1, None),
+    "accel_x": ("accel_z", "accel", 1, None),
+    "accel_y": ("accel_x", "accel", -1, None),
+    "accel_z": ("accel_y", "accel", -1, None),
+    "anglvel_x": ("gyro_z", "anglvel", 1, None),
     "anglvel_y": ("gyro_x", "anglvel", -1, None),
     "anglvel_z": ("gyro_y", "anglvel", -1, None),
-    "timestamp": ("gyro_ts", None, 1, None),
+    "timestamp": ("imu_ts", None, 1, None),
 }
 
-BTN_MAPPINGS: dict[int, str] = {
+BTN_MAPPINGS: dict[int, Button] = {
     # Volume buttons come from the same keyboard
     B("KEY_VOLUMEUP"): "key_volumeup",
     B("KEY_VOLUMEDOWN"): "key_volumedown",
     #
-    # AOKZOE A1 mappings And onexplayer mini pro mappings
-    #
-    # Turbo Button [29, 56, 125] KEY_LEFTCTRL + KEY_LEFTALT + KEY_LEFTMETA
-    B("KEY_LEFTALT"): "share",
-    # Short press orange [32, 125] KEY_D + KEY_LEFTMETA
-    B("KEY_D"): "mode",
-    # KB Button [24, 97, 125]  KEY_O + KEY_RIGHTCTRL + KEY_LEFTMETA
-    B("KEY_O"): "extra_l1",
-    #
-    # Loki Max
+    # Loki
     #
     B("KEY_T"): "share",  # T + LCTRL + LSHFT + LALT
 }
 
+AMBERNIC_MAPPINGS: dict[int, str] = {
+    B("KEY_LEFTMETA"): "share",
+    B("KEY_G"): "mode",
+}
+
+MSI_CLAW_MAPPINGS = {
+    B("KEY_F15"): "share",
+    B("KEY_F16"): "mode",
+}
+
 AYANEO_DEFAULT_MAPPINGS: dict[str, tuple[Axis, str | None, float, float | None]] = {
-    "accel_x": ("accel_z", "accel", 1, 3),
-    "accel_y": ("accel_x", "accel", 1, 3),
-    "accel_z": ("accel_y", "accel", 1, 3),
-    "anglvel_x": ("gyro_z", "anglvel", -1, None),
+    "accel_x": ("accel_z", "accel", 1, None),
+    "accel_y": ("accel_x", "accel", -1, None),
+    "accel_z": ("accel_y", "accel", -1, None),
+    "anglvel_x": ("gyro_z", "anglvel", 1, None),
     "anglvel_y": ("gyro_x", "anglvel", -1, None),
     "anglvel_z": ("gyro_y", "anglvel", -1, None),
-    "timestamp": ("gyro_ts", None, 1, None),
+    "timestamp": ("imu_ts", None, 1, None),
 }
 
 AYANEO_AIR_PLUS_MAPPINGS: dict[str, tuple[Axis, str | None, float, float | None]] = {
-    "accel_x": ("accel_z", "accel", 1, 3),
-    "accel_y": ("accel_x", "accel", 1, 3),
-    "accel_z": ("accel_y", "accel", 1, 3),
-    "anglvel_x": ("gyro_z", "anglvel", 1, None),
+    "accel_x": ("accel_z", "accel", -1, None),
+    "accel_y": ("accel_x", "accel", -1, None),
+    "accel_z": ("accel_y", "accel", 1, None),
+    "anglvel_x": ("gyro_z", "anglvel", -1, None),
     "anglvel_y": ("gyro_x", "anglvel", -1, None),
     "anglvel_z": ("gyro_y", "anglvel", 1, None),
-    "timestamp": ("gyro_ts", None, 1, None),
+    "timestamp": ("imu_ts", None, 1, None),
 }
 
 AYANEO_BTN_MAPPINGS: dict[int, str] = {
@@ -68,11 +69,11 @@ AYANEO_BTN_MAPPINGS: dict[int, str] = {
     # 2021 Mappings
     B("KEY_DELETE"): "share",  # TM Button [97,100,111]
     B("KEY_ESC"): "extra_l1",  # ESC Button [1]
-    B("KEY_O"): "extra_l2",  # KB Button [97, 24, 125]
+    B("KEY_O"): "extra_r1",  # KB Button [97, 24, 125]
     # B("KEY_LEFTMETA"): "extra_r1", # Win Button [125], Conflict with KB Button
     # Air mappings
     B("KEY_F11"): "extra_l1",  # LC Button [87, 97, 125] F11 + LCTRL + LMETA
-    B("KEY_F10"): "extra_l2",  # Rc Button [68, 97, 125] F10 + LCTRL + LMETA
+    B("KEY_F10"): "extra_r1",  # Rc Button [68, 97, 125] F10 + LCTRL + LMETA
 }
 
 AYA_DEFAULT_CONF = {
@@ -80,18 +81,29 @@ AYA_DEFAULT_CONF = {
     "btn_mapping": AYANEO_BTN_MAPPINGS,
     "mapping": AYANEO_DEFAULT_MAPPINGS,
 }
+ONEX_DEFAULT_CONF = {
+    "hrtimer": True,
+}
 
 CONFS = {
-    # Aokzoe
-    "AOKZOE A1 AR07": {"name": "AOKZOE A1", "hrtimer": True},
-    "AOKZOE A1 Pro": {"name": "AOKZOE A1 Pro", "hrtimer": True},
-    # Onexplayer
-    "ONEXPLAYER Mini Pro": {"name": "ONEXPLAYER Mini Pro", "hrtimer": True},
     # Ayn
+    "Loki MiniPro": {
+        "name": "Loki MiniPro",
+        "hrtimer": True,
+        "mapping": gen_gyro_state("x", False, "z", False, "y", True),
+        "extra_buttons": "none",
+    },
     "Loki Max": {
         "name": "Loki Max",
         "hrtimer": True,
-        "mapping": gen_gyro_state("x", False, "z", True, "y", True),
+        "mapping": gen_gyro_state("x", False, "z", False, "y", True),
+        "extra_buttons": "none",
+    },
+    "Loki Zero": {
+        "name": "Loki Zero",
+        "hrtimer": True,
+        "mapping": gen_gyro_state("x", False, "z", False, "y", True),
+        "extra_buttons": "none",
     },
     # Ayaneo
     "AIR Plus": {
@@ -114,7 +126,28 @@ CONFS = {
     "SLIDE": {
         "name": "AYANEO SLIDE",
         **AYA_DEFAULT_CONF,
-        "mapping": gen_gyro_state("z", True, "x", False, "y", False),
+        "mapping": gen_gyro_state("z", False, "x", False, "y", False),
+    },
+    "AYA NEO FOUNDER": {"name": "AYA NEO FOUNDER", **AYA_DEFAULT_CONF},
+    "AYA NEO 2021": {"name": "AYA NEO 2021", **AYA_DEFAULT_CONF},
+    "AYANEO 2021": {"name": "AYANEO 2021", **AYA_DEFAULT_CONF},
+    "AYANEO 2021 Pro": {"name": "AYANEO 2021 Pro", **AYA_DEFAULT_CONF},
+    "AYANEO 2021 Pro Retro Power": {
+        "name": "AYANEO 2021 Pro Retro Power",
+        **AYA_DEFAULT_CONF,
+    },
+    # Ambernic
+    "Win600": {
+        "name": "Ambernic Win600",
+        "btn_mapping": AMBERNIC_MAPPINGS,
+        "extra_buttons": "none",
+    },
+    # MSI Claw
+    "Claw A1M": {
+        "name": "MSI Claw (1st gen)",
+        # "hrtimer": True, Uses sensor fusion garbage? From intel? Will need custom work
+        "extra_buttons": "none",
+        "btn_mapping": MSI_CLAW_MAPPINGS,
     },
 }
 
