@@ -17,21 +17,21 @@ GPD_TOUCHPAD_AXIS_MAP: dict[int, Axis] = to_map(
 )
 
 GPD_WIN_DEFAULT_MAPPINGS: dict[str, tuple[Axis, str | None, float, float | None]] = {
-    "accel_x": ("accel_z", "accel", 1, 3),
-    "accel_y": ("accel_x", "accel", 1, 3),
-    "accel_z": ("accel_y", "accel", 1, 3),
+    "accel_x": ("accel_x", "accel", 1, None),
+    "accel_y": ("accel_z", "accel", 1, None),
+    "accel_z": ("accel_y", "accel", -1, None),
     "anglvel_x": ("gyro_x", "anglvel", 1, None),
-    "anglvel_y": ("gyro_z", "anglvel", -1, None),
+    "anglvel_y": ("gyro_z", "anglvel", 1, None),
     "anglvel_z": ("gyro_y", "anglvel", -1, None),
-    "timestamp": ("gyro_ts", None, 1, None),
+    "timestamp": ("imu_ts", None, 1, None),
 }
 
 GPD_WIN_MAX_2_2023_MAPPINGS: dict[str, tuple[Axis, str | None, float, float | None]] = {
-    "accel_x": ("accel_z", "accel", 1, 3),
-    "accel_y": ("accel_x", "accel", 1, 3),
-    "accel_z": ("accel_y", "accel", 1, 3),
-    "anglvel_x": ("gyro_z", "anglvel", 1, None),
+    "accel_x": ("accel_z", "accel", -1, None),
+    "accel_y": ("accel_x", "accel", -1, None),
+    "accel_z": ("accel_y", "accel", -1, None),
+    "anglvel_x": ("gyro_z", "anglvel", -1, None),
     "anglvel_y": ("gyro_x", "anglvel", -1, None),
     "anglvel_z": ("gyro_y", "anglvel", -1, None),
-    "timestamp": ("gyro_ts", None, 1, None),
+    "timestamp": ("imu_ts", None, 1, None),
 }
