@@ -4,18 +4,23 @@ import logging
 import os
 from copy import deepcopy
 from http.server import BaseHTTPRequestHandler, HTTPServer
+from socketserver import ThreadingMixIn
 from threading import Condition, Thread
-from typing import Any, Mapping
+from typing import Any, Mapping, Sequence, cast
 from urllib.parse import parse_qs, urlparse
 
 from hhd.plugins import (
     Config,
+    Context,
     Emitter,
+    HHDLocale,
     HHDSettings,
     get_relative_fn,
     load_relative_yaml,
 )
 
+from .i18n import get_user_lang, translate, translate_ver
+
 logger = logging.getLogger(__name__)
 
 
@@ -62,7 +67,9 @@ def parse_path(path: str) -> tuple[list, dict[str, list[str]]]:
         else:
             segments = []
 
-        params = {k: v for k, v in parse_qs(url.query).items() if v}
+        params = {
+            k: v for k, v in parse_qs(url.query, keep_blank_values=True).items() if v
+        }
         return segments, params
     except Exception:
         return [], {}
@@ -72,8 +79,12 @@ class RestHandler(BaseHTTPRequestHandler):
     settings: HHDSettings
     cond: Condition
     conf: Config
+    info: Config
     profiles: Mapping[str, Config]
     emit: Emitter
+    locales: Sequence[HHDLocale]
+    ctx: Context
+    user_lang: str | None
     token: str | None
 
     def set_response(self, code: int, headers: dict[str, str] = {}):
@@ -230,6 +241,9 @@ class RestHandler(BaseHTTPRequestHandler):
 
     def v1_endpoint(self, content: Any | None):
         segments, params = parse_path(self.path)
+        langs = params.get("lang", params.get("locale", None))
+        lang = langs[0] if langs else None
+
         if not segments:
             return self.send_not_found(f"Empty path.")
 
@@ -251,14 +265,19 @@ class RestHandler(BaseHTTPRequestHandler):
             case "profile":
                 self.handle_profile(segments[3:], params, content)
             case "settings":
-                v = self.conf.get("version", "")
+                v = translate_ver(self.conf, lang=lang, user_lang=self.user_lang)
                 self.set_response_ok({"Version": v})
                 with self.cond:
                     s = dict(deepcopy(self.settings))
                     try:
-                        s["hhd"]["version"] = {"type": "version", "value": v}  # type: ignore
+                        s["hhd"]["version"] = {  # type: ignore
+                            "type": "version",
+                            "tags": ["non-essential", "advanced", "expert", "hide"],
+                            "value": v,
+                        }
                     except Exception as e:
                         logger.error(f"Error while writing version hash to response.")
+                    s = translate(s, self.conf, self.locales, lang=lang, user_lang=self.user_lang)
                     self.wfile.write(json.dumps(s).encode())
             case "state":
                 self.set_response_ok()
@@ -270,11 +289,17 @@ class RestHandler(BaseHTTPRequestHandler):
                             )
                         self.emit({"type": "state", "config": Config(content)})
                         self.cond.wait()
-                    self.wfile.write(json.dumps(self.conf.conf).encode())
+                    elif "poll" in params:
+                        # Hang for the next update if the UI requests it.
+                        self.cond.wait()
+                    out = {**cast(dict, self.conf.conf), "info": self.info.conf}
+                    out["version"] = translate_ver(self.conf, lang=lang, user_lang=self.user_lang)
+                    out = translate(out, self.conf, self.locales, lang=lang, user_lang=self.user_lang)
+                    self.wfile.write(json.dumps(out).encode())
             case "version":
-                self.send_json({"version": 4})
+                self.send_json({"version": 5})
             case "sections":
-                self.send_json(SECTIONS)
+                self.send_json(translate(SECTIONS, self.conf, self.locales, lang=lang, user_lang=self.user_lang))
             case other:
                 self.send_not_found(f"Command '{other}' not supported.")
 
@@ -297,8 +322,12 @@ class RestHandler(BaseHTTPRequestHandler):
                 case other:
                     return self.send_not_found(f"File not found:\n{path}")
         except Exception as e:
-            logger.error(f"Encountered error while serving unauthenticated request.")
-            return self.send_error(f"Encountered error while serving request:\n{e}")
+            logger.debug(f"Encountered error while serving unauthenticated request.")
+            try:
+                return self.send_error(f"Encountered error while serving request:\n{e}")
+            except Exception:
+                # Generated due to polling website going in the background
+                pass
 
     def do_POST(self):
         if not self.send_authenticate():
@@ -333,6 +362,10 @@ class RestHandler(BaseHTTPRequestHandler):
         return self.do_GET
 
 
+class ThreadingSimpleServer(ThreadingMixIn, HTTPServer):
+    pass
+
+
 class HHDHTTPServer:
     def __init__(
         self,
@@ -359,18 +392,27 @@ class HHDHTTPServer:
         self,
         settings: HHDSettings,
         conf: Config,
+        info: Config,
         profiles: Mapping[str, Config],
         emit: Emitter,
+        locales: Sequence[HHDLocale],
+        ctx: Context,
     ):
         with self.cond:
             self.handler.settings = settings
             self.handler.conf = conf
+            self.handler.info = info
             self.handler.profiles = profiles
             self.handler.emit = emit
+            self.handler.locales = locales
+            self.handler.ctx = ctx
+            if not hasattr(self.handler, "user_lang"):
+                # Only load user lang once to avoid weirdness
+                self.handler.user_lang = get_user_lang(ctx)
             self.cond.notify_all()
 
     def open(self):
-        self.https = HTTPServer(
+        self.https = ThreadingSimpleServer(
             ("127.0.0.1" if self.localhost else "", self.port), self.handler
         )
         self.t = Thread(target=self.https.serve_forever)
