@@ -283,6 +283,7 @@ DEFAULT_TAGS = {
     "type": None,
     "title": "",
     "hint": "",
+    "unit": "",
     "tags": [],
     "default": None,
 }
@@ -725,7 +726,7 @@ def validate_config(
                         conf[k] = default
                     else:
                         del conf[k]
-            case "integer":
+            case "int" | "integer":
                 if not isinstance(v, int):
                     conf[k] = int(v)
                 if v < d["min"]:
